/* This file was generated by syncqt. */
#ifndef QT_QTQMLLINT_VERSION_H
#define QT_QTQMLLINT_VERSION_H

#define QTQMLLINT_VERSION_STR "6.12.0"

#define QTQMLLINT_VERSION 0x0601200

#endif // QT_QTQMLLINT_VERSION_H
