// Copyright (C) 2026 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
// Qt-Security score:significant reason:default

#ifndef QQUICKSHAPESDESIGNHELPERSGLOBAL_H
#define QQUICKSHAPESDESIGNHELPERSGLOBAL_H

#include <QtQml/qtqmlglobal.h>
#include <QtQuickShapesDesignHelpers/qtquickshapesdesignhelpersexports.h>

#endif // QQUICKSHAPESDESIGNHELPERSGLOBAL_H
